@@ -1,14 +1,10 @@
-use gtk::prelude::*;
-use gtk::{
-    gdk,
-    glib::{self, clone},
-    subclass::prelude::*,
-};
+use gtk::{gdk, glib, prelude::*, subclass::prelude::*};
 
 mod imp {
-    use super::*;
     use std::cell::{Cell, RefCell};
 
+    use super::*;
+
     #[derive(Default, Debug, gtk::CompositeTemplate)]
     #[template(resource = "/org/gnome/Tour/ui/paginator.ui")]
     pub struct PaginatorWidget {
@@ -33,7 +29,8 @@ mod imp {
         type Interfaces = (gtk::Buildable,);
 
         fn class_init(klass: &mut Self::Class) {
-            Self::bind_template(klass);
+            klass.bind_template();
+            klass.bind_template_instance_callbacks();
         }
 
         fn instance_init(obj: &glib::subclass::InitializingObject<Self>) {
@@ -42,48 +39,21 @@ mod imp {
     }
 
     impl ObjectImpl for PaginatorWidget {
-        fn constructed(&self, obj: &Self::Type) {
-            let layout_manager = obj
-                .layout_manager()
-                .map(|l| l.downcast::<gtk::BoxLayout>().unwrap())
-                .unwrap();
-            layout_manager.set_orientation(gtk::Orientation::Vertical);
+        fn constructed(&self) {
+            self.parent_constructed();
             self.carousel
                 .set_scroll_params(&adw::SpringParams::new(1.0, 0.5, 300.0));
-            self.carousel
-                .connect_position_notify(clone!(@weak obj => move |_| {
-                    obj.update_position();
-                }));
-
-            let controller = gtk::EventControllerKey::new();
-            controller.connect_key_pressed(clone!(@weak obj => @default-return gtk::Inhibit(true), move |_controller, keyval, _keycode, _state| {
-                if keyval == gdk::Key::Right {
-                    obj.try_next();
-                } else if keyval == gdk::Key::Left {
-                    obj.try_previous();
-                }
-                gtk::Inhibit(false)
-            }));
-
-            obj.add_controller(&controller);
-
-            self.parent_constructed(obj);
         }
     }
     impl WidgetImpl for PaginatorWidget {}
     impl BoxImpl for PaginatorWidget {}
     impl BuildableImpl for PaginatorWidget {
-        fn add_child(
-            &self,
-            buildable: &Self::Type,
-            builder: &gtk::Builder,
-            child: &glib::Object,
-            type_: Option<&str>,
-        ) {
+        fn add_child(&self, builder: &gtk::Builder, child: &glib::Object, type_: Option<&str>) {
             if !self.carousel.is_bound() {
-                self.parent_add_child(buildable, builder, child, type_);
+                self.parent_add_child(builder, child, type_);
             } else {
-                buildable.add_page(child.clone().downcast::<gtk::Widget>().unwrap());
+                self.obj()
+                    .add_page(child.clone().downcast::<gtk::Widget>().unwrap());
             }
         }
     }
@@ -95,11 +65,8 @@ glib::wrapper! {
         @implements gtk::Buildable;
 }
 
+#[gtk::template_callbacks]
 impl PaginatorWidget {
-    pub fn new() -> Self {
-        glib::Object::new(&[]).unwrap()
-    }
-
     pub fn try_next(&self) -> Option<()> {
         let imp = self.imp();
         let p = imp.current_page.get() + 1;
@@ -119,16 +86,27 @@ impl PaginatorWidget {
         Some(())
     }
 
-    pub fn add_page(&self, page: impl IsA<gtk::Widget>) {
+    fn add_page(&self, page: impl IsA<gtk::Widget>) {
         let imp = self.imp();
         let page_nr = imp.pages.borrow().len();
         imp.carousel.insert(&page, page_nr as i32);
         imp.pages.borrow_mut().push(page.upcast());
 
-        self.update_position();
+        self.on_position_notify();
     }
 
-    fn update_position(&self) {
+    #[template_callback]
+    fn on_key_pressed(&self, keyval: gdk::Key) -> gtk::Inhibit {
+        if keyval == gdk::Key::Right {
+            self.try_next();
+        } else if keyval == gdk::Key::Left {
+            self.try_previous();
+        }
+        gtk::Inhibit(false)
+    }
+
+    #[template_callback]
+    fn on_position_notify(&self) {
         let imp = self.imp();
 
         let position = imp.carousel.position();
@@ -194,9 +172,3 @@ impl PaginatorWidget {
         }
     }
 }
-
-impl Default for PaginatorWidget {
-    fn default() -> Self {
-        Self::new()
-    }
-}
