@@ -1,16 +1,12 @@
-use crate::utils::i18n_f;
-use adw::prelude::*;
-use gtk::subclass::prelude::*;
+use adw::{prelude::*, subclass::prelude::*};
 use gtk::{gio, glib};
 
 use super::paginator::PaginatorWidget;
-use crate::Application;
+use crate::{utils::i18n_f, Application};
 
 mod imp {
     use super::*;
-    use crate::config;
-    use crate::widgets::ImagePageWidget;
-    use adw::subclass::prelude::*;
+    use crate::{config, widgets::ImagePageWidget};
 
     #[derive(Debug, Default, gtk::CompositeTemplate)]
     #[template(resource = "/org/gnome/Tour/ui/window.ui")]
@@ -29,6 +25,24 @@ mod imp {
 
         fn class_init(klass: &mut Self::Class) {
             klass.bind_template();
+            // Start Tour
+            klass.install_action("win.start-tour", None, |win, _, _| win.start_tour());
+            // Skip Tour
+            klass.install_action("win.skip-tour", None, |win, _, _| {
+                win.application().unwrap().quit();
+            });
+            // Next page
+            klass.install_action("win.next-page", None, |win, _, _| {
+                if win.imp().paginator.try_next().is_none() {
+                    win.close();
+                }
+            });
+            // Previous page
+            klass.install_action("win.previous-page", None, |win, _, _| {
+                if win.imp().paginator.try_previous().is_none() {
+                    win.reset_tour();
+                }
+            });
         }
 
         fn instance_init(obj: &glib::subclass::InitializingObject<Self>) {
@@ -37,7 +51,9 @@ mod imp {
     }
 
     impl ObjectImpl for Window {
-        fn constructed(&self, widget: &Self::Type) {
+        fn constructed(&self) {
+            self.parent_constructed();
+            let widget = self.obj();
             widget.set_icon_name(Some(config::APP_ID));
 
             // Devel Profile
@@ -48,12 +64,12 @@ mod imp {
             let version = glib::os_info("VERSION").unwrap_or_else(|| "".into());
 
             let body = i18n_f(
-                // Translators: The following string is formated as "Learn about new and essential features in GNOME 3.36" for example
+                // Translators: The following string is formated as "Learn about new and essential
+                // features in GNOME 3.36" for example
                 "Learn about the key features in {name} {version}.",
                 &[("name", &name), ("version", &version)],
             );
-            self.welcome_page.set_body(&body);
-            self.parent_constructed(widget);
+            self.welcome_page.set_body(body);
         }
     }
     impl WidgetImpl for Window {}
@@ -70,11 +86,7 @@ glib::wrapper! {
 
 impl Window {
     pub fn new(app: &Application) -> Self {
-        glib::Object::new(&[("application", app)]).unwrap()
-    }
-
-    pub fn paginator(&self) -> PaginatorWidget {
-        self.imp().paginator.clone()
+        glib::Object::builder().property("application", app).build()
     }
 
     pub fn start_tour(&self) {
